@@ -25,6 +25,8 @@
 #
 
 
+from __future__ import unicode_literals
+
 import os
 import logging
 import logging.handlers
@@ -36,12 +38,14 @@ import poster
 import time
 #from datetime import datetime
 import datetime
-from progressbar import *
+#from progressbar import *
+from progressbar import ProgressBar, FileTransferSpeed, Bar, ETA, Percentage
 import hashlib
-
+import locale
 
 # -----------------------------------------------------------------------------
 def extract_file_name(content_dispo):
+    """Extract file name from the input request body"""
     #print type(content_dispo)
     #print repr(content_dispo)
     file_name = ""
@@ -49,13 +53,14 @@ def extract_file_name(content_dispo):
         param = key_val.strip().split('=')
         if param[0] == "filename":
             file_name = param[1]
-            # convertion of escape string (str type) from server to unicode object
+            # convertion of escape string (str type) from server
+            # to unicode object
             file_name = file_name.decode('unicode-escape').strip('"')
             break
     return file_name
 
 
-class file_with_callback(file):
+class FileWithCallback(file):
     def __init__(self, path, mode, callback, size=None, *args):
         file.__init__(self, path, mode)
         if size:
@@ -105,7 +110,7 @@ def cli_get_cache(user_function):
         log.debug("function time : " + str(end - start))
         return res
 
-    def get_data(func, cachefile,  *args):
+    def get_data(func, cachefile, *args):
         res = log_exec_time(func, *args)
         with open(cachefile, 'wb') as f:
             json.dump(res, f)
@@ -207,11 +212,16 @@ class CoreCli(object):
 
         # We declare all the handlers useful here.
         auth_handler = urllib2.HTTPBasicAuthHandler()
-        auth_handler.add_password(
-            realm=realm,
-            uri=host,
-            user=user,
-            passwd=password)
+        # we convert unicode objects to utf8 strings because
+        # the authentication module does not handle unicode
+        try:
+            auth_handler.add_password(
+                realm=realm.encode('utf8'),
+                uri=host.encode('utf8'),
+                user=user.encode('utf8'),
+                passwd=password.encode('utf8'))
+        except UnicodeEncodeError as ex:
+            self.log.error("the program was not able to compute the basic authentication token.")
 
         handlers = [
             poster.streaminghttp.StreamingHTTPSHandler(
@@ -260,7 +270,7 @@ class CoreCli(object):
         #request.add_header("Cookie", "JSESSIONID=")
 
     @cli_get_cache
-    def _list(self, url):
+    def list(self, url):
         """ List all documents store into LinShare."""
         self.last_req_time = None
         url = self.root_url + url
@@ -286,6 +296,7 @@ class CoreCli(object):
                 self.log.debug("the result is : ")
                 self.log.debug(json.dumps(jObj, sort_keys=True, indent=2))
         else:
+            self.log.debug(str(result))
             msg = "Wrong content type in the http response " + content_type
             self.log.error(msg)
             raise ValueError(msg)
@@ -296,7 +307,7 @@ class CoreCli(object):
                         "time": self.last_req_time})
         return jObj
 
-    def _delete(self, url):
+    def delete(self, url):
         """ List all documents store into LinShare."""
         self.last_req_time = None
         url = self.root_url + url
@@ -336,30 +347,29 @@ class CoreCli(object):
                         "time": self.last_req_time})
         return jObj
 
-    def _upload(self, file_path, url, description = None):
+    def upload(self, file_path, url, description=None):
         self.last_req_time = None
         url = self.root_url + url
         self.log.debug("upload url : " + url)
 
         # Generating datas and headers
         file_size = os.path.getsize(file_path)
+
+        file_path = file_path.decode(locale.getpreferredencoding())
         self.log.debug("file_path is : " + file_path)
         file_name = os.path.basename(file_path)
         self.log.debug("file_name is : " + file_name)
 
-        file_name = file_name.decode('UTF-8')
-        file_path = file_path.decode('UTF-8')
-
         if file_size <= 0:
             self.log.fatal("""The file '%(filename)s' can not be uploaded
-                because its size less or equal to zero.""",
+because its size less or equal to zero.""",
                            {"filename": str(file_name)})
             return None
 
         widgets = [FileTransferSpeed(), ' <<<', Bar(), '>>> ',
                    Percentage(), ' ', ETA()]
         pbar = ProgressBar(widgets=widgets, maxval=file_size)
-        stream = file_with_callback(file_path, 'rb', pbar.update,
+        stream = FileWithCallback(file_path, 'rb', pbar.update,
                                     file_size, file_path)
 
 
@@ -420,7 +430,7 @@ class CoreCli(object):
                         "time": self.last_req_time})
         return jObj
 
-    def _download(self, uuid, url):
+    def download(self, uuid, url):
         """ download a file from LinShare using its rest api.
 This method could throw exceptions like urllib2.HTTPError."""
         self.last_req_time = None
@@ -462,7 +472,7 @@ This method could throw exceptions like urllib2.HTTPError."""
             widgets = [FileTransferSpeed(), ' <<<', Bar(), '>>> ',
                        Percentage(), ' ', ETA()]
             pbar = ProgressBar(widgets=widgets, maxval=file_size)
-            stream = file_with_callback(file_name, 'w', pbar.update,
+            stream = FileWithCallback(file_name, 'w', pbar.update,
                                         file_size, file_name)
             pbar.start()
 
@@ -498,20 +508,20 @@ class Documents(object):
 
     def list(self):
         """ List all documents store into LinShare."""
-        return self.core._list("documents.json")
+        return self.core.list("documents.json")
 
-    def upload(self, file_path, description = None):
+    def upload(self, file_path, description=None):
         """ Upload a file to LinShare using its rest api.
         The uploaded document uuid will be returned"""
-        return self.core._upload(file_path, "documents.json", description)
+        return self.core.upload(file_path, "documents.json", description)
 
     def download(self, uuid):
         url = "documents/%s/download" % uuid
-        return self.core._download(uuid, url)
+        return self.core.download(uuid, url)
 
     def delete(self, uuid):
         url = "documents/%s.json" % uuid
-        return self.core._delete(url)
+        return self.core.delete(url)
 
 
 
@@ -521,11 +531,11 @@ class ReceivedShares(object):
         self.core = corecli
 
     def list(self):
-        return self.core._list("shares.json")
+        return self.core.list("shares.json")
 
     def download(self, uuid):
         url = "shares/%s/download" % uuid
-        return self.core._download(uuid, url)
+        return self.core.download(uuid, url)
 
 
 # -----------------------------------------------------------------------------
@@ -574,7 +584,7 @@ class Threads(object):
         self.core = corecli
 
     def list(self):
-        return self.core._list("threads.json")
+        return self.core.list("threads.json")
 
 
 # -----------------------------------------------------------------------------
@@ -584,7 +594,7 @@ class ThreadsMembers(object):
 
     def list(self, threadUuid):
         url = "thread_members/%s.json" % threadUuid
-        return self.core._list(url)
+        return self.core.list(url)
 
 
 # -----------------------------------------------------------------------------
@@ -593,7 +603,7 @@ class Users(object):
         self.core = corecli
 
     def list(self):
-        return self.core._list("users.json")
+        return self.core.list("users.json")
 
 
 # -----------------------------------------------------------------------------
@@ -616,7 +626,7 @@ class ThreadsAdmin(object):
         self.core = corecli
 
     def list(self):
-        return self.core._list("admin/threads.json")
+        return self.core.list("admin/threads.json")
 
 
 # -----------------------------------------------------------------------------
