@@ -24,15 +24,20 @@
 #  Frédéric MARTIN frederic.martin.fma@gmail.com
 #
 
+from __future__ import unicode_literals
+
 import logging
 import urllib2
 import copy
 import os
+import locale
 import linshare_cli.common as common
 from linshare_cli.core import UserCli
 from argtoolbox import DefaultCompleter
 from argtoolbox import query_yes_no
 import argtoolbox
+from argparse import RawTextHelpFormatter
+from veryprettytable import VeryPrettyTable
 
 
 # -----------------------------------------------------------------------------
@@ -59,6 +64,9 @@ class TestCommand(argtoolbox.DefaultCommand):
         self.verbose = args.verbose
         self.debug = args.debug
         print "Test"
+        print unicode(self.config)
+        print args
+        print ""
 
 
 # -----------------------------------------------------------------------------
@@ -113,7 +121,6 @@ configure  it :
 
 """
 
-
 # -------------------------- Documents ----------------------------------------
 # -----------------------------------------------------------------------------
 class DocumentsListCommand(DefaultCommand):
@@ -123,11 +130,119 @@ class DocumentsListCommand(DefaultCommand):
         super(DocumentsListCommand, self).__call__(args)
 
         json_obj = self.ls.documents.list()
-        d_format = u"{name:60s}{creationDate:30s}{uuid:40s}{description}"
+        if args.show_columns:
+            self.print_fields(json_obj)
+            return True
+
+        keys = []
+        keys.append(u'name')
+        keys.append(u'size')
+        keys.append(u'uuid')
+        keys.append(u'creationDate')
+
+        if args.extended:
+            keys.append(u'type')
+            keys.append(u'expirationDate')
+            keys.append(u'modificationDate')
+
         self.format_date(json_obj, 'creationDate')
+        self.format_filesize(json_obj, 'size')
+        self.format_date(json_obj, 'modificationDate')
+        self.format_date(json_obj, 'expirationDate')
+
+        # computing data for presentation
+        maxlength = self.getmaxlength(json_obj)
+        datatype = self.getdatatype(json_obj)
+
+        # computing string format
+        d_format = ""
+        if args.output_format:
+            d_format = args.output_format
+            d_format = d_format.decode(locale.getpreferredencoding())
+        else:
+            for key in keys:
+                d_format += self.build_on_field(key, maxlength, datatype)
+
         from operator import itemgetter
-        json_obj = sorted(json_obj, reverse = True, key=itemgetter("creationDate"))
-        self.print_list(json_obj, d_format, "Documents")
+        param = "creationDate"
+        reverse = args.reverse
+        if args.name:
+            param = "name"
+        if args.size:
+            param = "size_raw"
+
+        json_obj = sorted(json_obj, reverse=reverse, key=itemgetter(param))
+        if args.no_title:
+            self.print_list(json_obj, d_format, no_legend=args.no_legend)
+        else:
+            self.print_list(json_obj, d_format, "Documents",
+                            no_legend=args.no_legend)
+
+
+# -------------------------- Documents ----------------------------------------
+# -----------------------------------------------------------------------------
+class DocumentsListV2Command(DefaultCommand):
+    """ List all documents store into LinShare."""
+
+    def __call__(self, args):
+        super(DocumentsListV2Command, self).__call__(args)
+
+        json_obj = self.ls.documents.list()
+
+        keys = []
+        keys.append(u'name')
+        keys.append(u'uuid')
+        keys.append(u'size')
+        keys.append(u'creationDate')
+        if args.extended:
+            keys.append(u'type')
+            keys.append(u'expirationDate')
+            keys.append(u'description')
+            keys.append(u'modificationDate')
+            keys.append(u'ciphered')
+
+        self.format_date(json_obj, 'creationDate')
+        self.format_filesize(json_obj, 'size')
+        self.format_date(json_obj, 'modificationDate')
+        self.format_date(json_obj, 'expirationDate')
+
+        table = VeryPrettyTable(keys)
+
+        # styles
+        table.align[u"name"] = "l"
+        table.padding_width = 1
+        table.align["size"] = "l"
+
+        if args.name:
+            table.reversesort = args.reverse
+            table.sortby = "name"
+        elif args.size:
+            from operator import itemgetter
+            json_obj = sorted(json_obj, reverse=args.reverse,
+                              key=itemgetter("size_raw"))
+        else:
+            table.reversesort = args.reverse
+            table.sortby = "creationDate"
+
+        if True:
+            for row in json_obj:
+                data = []
+                for key in keys:
+                    data.append(row[key])
+                table.add_row(data)
+                #table.add_row(data, fore_color="red")
+        if False:
+            table = VeryPrettyTable()
+            for key in keys:
+                table.add_column(key, [row[key] for row in json_obj])
+
+        out = table.get_string(
+            fields=keys,
+            #start=10,
+            #end=10,
+            #sortby=param
+            )
+        print unicode(out)
 
 
 # -----------------------------------------------------------------------------
@@ -164,7 +279,7 @@ class DocumentsDownloadCommand(DefaultCommand):
                 print ex
                 return
 
-    def complete(self, args,  prefix):
+    def complete(self, args, prefix):
         super(DocumentsDownloadCommand, self).__call__(args)
 
         json_obj = self.ls.documents.list()
@@ -191,7 +306,7 @@ class DocumentsDeleteCommand(DefaultCommand):
                 print ex
                 return
 
-    def complete(self, args,  prefix):
+    def complete(self, args, prefix):
         super(DocumentsDeleteCommand, self).__call__(args)
 
         json_obj = self.ls.documents.list()
@@ -228,12 +343,50 @@ class DocumentsUploadAndSharingCommand(DefaultCommand):
                     self.log.error("Unexpected return code : " +
                                    str(code) + " : " + msg)
 
-    def complete(self, args,  prefix):
+    def complete(self, args, prefix):
         super(DocumentsUploadAndSharingCommand, self).__call__(args)
 
-        json_obj = self.ls.documents.list()
-        return (
-            v.get('uuid') for v in json_obj if v.get('uuid').startswith(prefix))
+        from argcomplete import warn
+        if len(prefix) >= 3:
+            json_obj = self.ls.users.list()
+            return (v.get('mail')
+                    for v in json_obj if v.get('mail').startswith(prefix))
+        else:
+            warn("Completion need at least 3 characters.")
+
+    def complete2(self, args, prefix):
+        super(DocumentsUploadAndSharingCommand, self).__call__(args)
+
+        from argcomplete import warn
+        if len(prefix) >= 3:
+            import re
+            json_obj = self.ls.users.list()
+            guesses = []
+            mails = []
+            cpt = 0
+            for v in json_obj:
+                mail = v.get('mail')
+                if re.match(".*" + prefix + ".*", mail):
+                    guesses.append(mail)
+                if mail.startswith(prefix):
+                    cpt += 1
+                    mails.append(mail)
+                if cpt >=5:
+                    break
+            if mails:
+                return mails
+            else:
+                cpt = 0
+                warning = ["Some results :"]
+                for i in guesses:
+                    cpt += 1
+                    warning.append(" * " + i + "\n")
+                    if cpt >=4:
+                        break
+                warn("".join(warning))
+                return guesses
+        else:
+            warn("Completion need at least 3 characters.")
 
 
 # ----------------- Received Shares -------------------------------------------
@@ -296,14 +449,14 @@ class SharesCommand(DefaultCommand):
                     self.log.error("Unexpected return code : " + str(code) +
                                    " : " + msg)
 
-    def complete(self, args,  prefix):
+    def complete(self, args, prefix):
         super(SharesCommand, self).__call__(args)
 
         json_obj = self.ls.documents.list()
         return (
             v.get('uuid') for v in json_obj if v.get('uuid').startswith(prefix))
 
-    def complete_mail(self, args,  prefix):
+    def complete_mail(self, args, prefix):
         super(SharesCommand, self).__call__(args)
 
         if len(prefix) >= 3:
@@ -342,7 +495,7 @@ class ThreadMembersListCommand(DefaultCommand):
         #self.pretty_json(json_obj)
         self.print_list(json_obj, d_format, "Thread members")
 
-    def complete(self, args,  prefix):
+    def complete(self, args, prefix):
         super(ThreadMembersListCommand, self).__call__(args)
 
         json_obj = self.ls.threads.list()
@@ -383,9 +536,14 @@ def add_document_parser(subparsers, name, desc):
     parser_tmp2 = subparsers2.add_parser('upshare',
                                          help="upload and share documents")
     parser_tmp2.set_defaults(__func__=DocumentsUploadAndSharingCommand())
-    parser_tmp2.add_argument('files', nargs='+').completer = DefaultCompleter()
-    parser_tmp2.add_argument('-m', '--mail', action="append", dest="mails",
-                             required=True, help="Recipient mails.")
+    parser_tmp2.add_argument('files', nargs='+')
+    parser_tmp2.add_argument('-m',
+                             '--mail',
+                             action="append",
+                             dest="mails",
+                             required=True,
+                             help="Recipient mails."
+                             ).completer = DefaultCompleter()
 
 
     parser_tmp2 = subparsers2.add_parser(
@@ -402,10 +560,50 @@ def add_document_parser(subparsers, name, desc):
 
     parser_tmp2 = subparsers2.add_parser(
         'list',
+        formatter_class=RawTextHelpFormatter,
         help="list documents from linshare")
+    parser_tmp2.add_argument('-r', '--reverse', action="store_true",
+                             help="reverse order while sorting")
+    parser_tmp2.add_argument('-c', '--creation-date', action="store_true",
+                             help="sort by creation date")
+    parser_tmp2.add_argument('-n', '--name', action="store_true",
+                             help="sort by file name")
+    parser_tmp2.add_argument('--extended', action="store_true",
+                             help="extended format")
+    parser_tmp2.add_argument('--no-title', action="store_true",
+                             help="dont show the title")
+    parser_tmp2.add_argument('--no-legend', action="store_true",
+                             help="dont show the legend")
+    parser_tmp2.add_argument('--size', action="store_true",
+                             help="sort by file size")
+    parser_tmp2.add_argument('-f', '--format', dest="output_format",
+                             action="store",
+                             help="""Change output format, ex:
+{name:60s}{size!s:^10s}{uuid:40s}""")
+    parser_tmp2.add_argument('--show-columns', action="store_true",
+                             help="List all available fields in received data.")
     parser_tmp2.set_defaults(__func__=DocumentsListCommand())
 
 
+    parser_tmp2 = subparsers2.add_parser(
+        'listv2',
+        formatter_class=RawTextHelpFormatter,
+        help="list documents from linshare")
+    parser_tmp2.add_argument('-r', '--reverse', action="store_true",
+                             help="reverse order while sorting", default=False)
+    parser_tmp2.add_argument('-c', '--creation-date', action="store_true",
+                             help="sort by creation date")
+    parser_tmp2.add_argument('-n', '--name', action="store_true",
+                             help="sort by file name")
+    #parser_tmp2.add_argument('-x', action="store_true",
+    #                         help="vertical display")
+    parser_tmp2.add_argument('--extended', action="store_true",
+                             help="extended format")
+    parser_tmp2.add_argument('--size', action="store_true",
+                             help="sort by file size")
+    parser_tmp2.set_defaults(__func__=DocumentsListV2Command())
+
+
 ###############################################################################
 ### shares
 ###############################################################################
@@ -520,6 +718,7 @@ def add_config_parser(subparsers, name, desc, config):
 ###############################################################################
 ### test
 ###############################################################################
-def add_test_parser(subparsers):
+def add_test_parser(subparsers, config):
     parser_tmp = subparsers.add_parser('test', add_help=False)
-    parser_tmp.set_defaults(__func__=TestCommand())
+    parser_tmp.add_argument('files', nargs='*')
+    parser_tmp.set_defaults(__func__=TestCommand(config))
